'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

exports.default = function (element, options) {

  if (!element) {
    throw 'Input element must be supplied as first argument';
  }

  var input = new Finput(element, options || {});
  element.setRawValue = function (v) {
    return input.setRawValue(v);
  };
  element.setValue = function (v) {
    return input.setValue(v);
  };
  element.getOptions = function () {
    return input.options;
  };
  element.setOptions = function (o) {
    return input.options = o;
  };

  return function () {
    input.removeListeners();
    delete element.setRawValue;
    delete element.setValue;
    delete element.getOptions;
    delete element.setOptions;
  };
};

var _keyHandlers = require('./keyHandlers');

var _keyHandlers2 = _interopRequireDefault(_keyHandlers);

var _helpers = require('./helpers');

var _helpers2 = _interopRequireDefault(_helpers);

var _valueHistory = require('./valueHistory');

var _valueHistory2 = _interopRequireDefault(_valueHistory);

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * CONSTANTS
 */
var DEFAULTS = {
  scale: 2,
  range: _constants.RANGE.ALL,
  fixed: true,
  thousands: ',',
  decimal: '.',
  shortcuts: {
    'k': 1000,
    'm': 1000000,
    'b': 1000000000
  },
  invalidKeyCallback: function invalidKeyCallback() {}

  /**
   * FINPUT COMPONENT CLASS
   * @class
   */
};
var Finput = function () {

  /**
   * Constructor
   * @param {DOM Element} The number input
   * @param {Options} Options for the number input's behaviour
   *
   * Detailed list of possible options:
   * @param {Options.scale} maximum number of decimal digits
   * @param {Options.range} Whether number can take any value or must be positive
   * @param {Options.fixed} After focus is lost - value is formatted to *scale* number of decimal places
   * @param {Options.thousands} Character to use for the thousands separator
   * @param {Options.decimal} Character to use for the decimal point
   * @param {Options.shortcuts} Object map of shortcut characters to multiplier (e.g. { k: 1000 })
   * @param {Options.invalidKeyCallback} Function callback that will be called on an invalid keypress
   * @param {Options.onFocusinCallback} Function callback that will be called via the onFocusin event
   */
  function Finput(element, options) {
    _classCallCheck(this, Finput);

    this._element = element;
    this._options = _extends({}, DEFAULTS, options);

    this._actionTypes = this.createActionTypes();
    this._history = new _valueHistory2.default();

    this._listeners = {
      blur: { element: this.element, handler: this.onFocusout.bind(this) },
      focus: { element: this.element, handler: this.onFocusin.bind(this) },
      drop: { element: this.element, handler: this.onDrop.bind(this) },
      paste: { element: this.element, handler: this.onPaste.bind(this) },
      keydown: { element: this.element, handler: this.onKeydown.bind(this) },
      input: { element: this.element, handler: this.onInput.bind(this) },

      dragstart: { element: document, handler: this.onDragstart.bind(this) },
      dragend: { element: document, handler: this.onDragend.bind(this) }

      // Setup listeners
    };this.removeListeners();
    for (var e in this._listeners) {
      this._listeners[e].element.addEventListener(e, this._listeners[e].handler);
    }
  }

  // GETTERS


  _createClass(Finput, [{
    key: 'createActionTypes',


    /**
     * Creates the correct action type to char/key codes array with the
     * correct decimal and thousand separator characters (depending on language)
     */
    value: function createActionTypes() {
      return [{
        type: _constants.ACTION_TYPES.NUMBER,
        names: ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9']
      }, {
        type: _constants.ACTION_TYPES.MINUS,
        names: ['-']
      }, {
        type: _constants.ACTION_TYPES.DECIMAL,
        names: [this.options.decimal]
      }, {
        type: _constants.ACTION_TYPES.THOUSANDS,
        names: [this.options.thousands]
      }, {
        type: _constants.ACTION_TYPES.SHORTCUT,
        names: Object.keys(this.options.shortcuts)
      }, {
        type: _constants.ACTION_TYPES.BACKSPACE,
        names: ['backspace']
      }, {
        type: _constants.ACTION_TYPES.DELETE,
        names: ['delete', // Chrome & Firefox
        'del' // Edge & IE
        ]
      }, {
        type: _constants.ACTION_TYPES.UNDO,
        names: ['z'],
        ctrl: true
      }, {
        type: _constants.ACTION_TYPES.REDO,
        names: ['y'],
        ctrl: true
      }];
    }

    /**
     * Determines what type of action needs to be dealt with from the current
     * keydown event. E.g. vertical arrow pressed, number pressed etc...
     * @param {e} Keyboard event
     */

  }, {
    key: 'getActionType',
    value: function getActionType(event) {
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = this._actionTypes[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var actionType = _step.value;

          var index = actionType.names.indexOf(event.keyName);
          var typeMatch = index > -1;

          if (typeMatch && (actionType.modifierKey ? event.modifierKey : true)) {
            return actionType.type;
          }
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      return _constants.ACTION_TYPES.UNKNOWN;
    }
  }, {
    key: 'getHandlerForAction',
    value: function getHandlerForAction(action) {
      var _handlerForAction;

      var handlerForAction = (_handlerForAction = {}, _defineProperty(_handlerForAction, _constants.ACTION_TYPES.NUMBER, _keyHandlers2.default.onNumber), _defineProperty(_handlerForAction, _constants.ACTION_TYPES.DECIMAL, _keyHandlers2.default.onDecimal), _defineProperty(_handlerForAction, _constants.ACTION_TYPES.THOUSANDS, _keyHandlers2.default.onThousands), _defineProperty(_handlerForAction, _constants.ACTION_TYPES.MINUS, _keyHandlers2.default.onMinus), _defineProperty(_handlerForAction, _constants.ACTION_TYPES.SHORTCUT, _keyHandlers2.default.onShortcut), _defineProperty(_handlerForAction, _constants.ACTION_TYPES.BACKSPACE, _keyHandlers2.default.onBackspace), _defineProperty(_handlerForAction, _constants.ACTION_TYPES.DELETE, _keyHandlers2.default.onDelete), _defineProperty(_handlerForAction, _constants.ACTION_TYPES.UNDO, _keyHandlers2.default.onUndo), _defineProperty(_handlerForAction, _constants.ACTION_TYPES.REDO, _keyHandlers2.default.onRedo), _defineProperty(_handlerForAction, _constants.ACTION_TYPES.UNKNOWN, _keyHandlers2.default.onUnknown), _handlerForAction);

      return handlerForAction[action];
    }

    /**
     * Get numerical value of the given value
     * @param {value} Value to convert
     */

  }, {
    key: 'getRawValue',
    value: function getRawValue(value) {
      return _helpers2.default.formattedToRaw(value, this.options);
    }

    /**
     * Sets the value, fully formatted, for the input
     * @param {val} New value to set
     * @param {notNull} When true, restricts setting the value if it is null.
     */

  }, {
    key: 'setValue',
    value: function setValue(val, notNull) {
      var newValue = _helpers2.default.fullFormat(val, this.options);

      if (notNull ? val : true) {
        this.element.value = newValue;
        this.element.rawValue = this.getRawValue(this.element.value);
        this._history.addValue(newValue);
      }
    }

    /**
     * Sets and formats the value for the input
     * @param {val} New value to set
     */

  }, {
    key: 'setRawValue',
    value: function setRawValue(val) {
      var value = void 0;
      if (!val) {
        value = '';
      } else if (typeof val === 'number' && !isNaN(val)) {
        value = _helpers2.default.rawToFormatted(val, this.options);
      } else if (typeof val === 'string') {
        value = val;
      } else {
        return;
      }

      var newValue = _helpers2.default.parseString(value, this.options);
      this.setValue(newValue, false);
    }

    /**
     * Returns whether a modifier key has been pressed
     * 
     * On macOS the Command (meta) key is the modifier key.
     * On Windows the Control (ctrl) key is the modifier key.
     * Note that on Windows the meta key is the Windows key.
     * 
     * @param {e} Key event 
     */

  }, {
    key: 'isModifierKeyPressed',
    value: function isModifierKeyPressed(e) {
      var isMac = navigator.platform.toUpperCase().indexOf('MAC') >= 0;
      return isMac ? e.metaKey : e.ctrlKey;
    }

    //
    // EVENT HANDLERS
    //

    /**
     * On focusing OUT of the input - format fully
     * @param {e} Focus event
     */

  }, {
    key: 'onFocusout',
    value: function onFocusout(e) {
      this.setValue(this.element.value);
    }
    /**
     * On focusing IN of the input
     * DEFAULT:  Select all text
     * @param {e} Focus event
     */

  }, {
    key: 'onFocusin',
    value: function onFocusin(e) {
      if (this.options.onFocusinCallback) {
        var selection = this.options.onFocusinCallback(e);
        if (selection) {
          this.element.selectionStart = selection.start;
          this.element.selectionEnd = selection.end;
        }
      } else {
        this.element.selectionStart = 0;
        this.element.selectionEnd = this.element.value.length;
      }
    }
    /**
     * On dropping something into the input - replace the WHOLE value
     * with this new value
     * @param {e} Drag event
     */

  }, {
    key: 'onDrop',
    value: function onDrop(e) {
      switch (this._dragState) {
        case _constants.DRAG_STATES.INTERNAL:
          // This case is handled by the 'onInput' function
          break;
        case _constants.DRAG_STATES.EXTERNAL:
          var val = _helpers2.default.parseString(e.dataTransfer.getData('text'), this.options);
          this.setValue(val, true);
          e.preventDefault();
          break;
        default:
          // Do nothing;
          break;
      }
    }

    /**
     * On start of ANY drag on page
     * @param {e} Drag event
     */

  }, {
    key: 'onDragstart',
    value: function onDragstart(e) {
      this._dragState = e.target === this.element ? _constants.DRAG_STATES.INTERNAL : _constants.DRAG_STATES.EXTERNAL;
    }
    /**
     * On end of ANY drag on page
     * @param {e} Drag event
     */

  }, {
    key: 'onDragend',
    value: function onDragend(e) {
      this._dragState = _constants.DRAG_STATES.NONE;
    }
    /**
     * On pasting something into the input
     * @param {e} Clipboard event
     */

  }, {
    key: 'onPaste',
    value: function onPaste(e) {
      var val = _helpers2.default.parseString(e.clipboardData.getData('text'), this.options);
      this.setValue(val, true);
      e.preventDefault();
    }

    /**
     * On pressing any key inside the input
     * @param {e} Keyboard event
     */

  }, {
    key: 'onKeydown',
    value: function onKeydown(e) {
      var currentState = {
        caretStart: this.element.selectionStart,
        caretEnd: this.element.selectionEnd,
        value: this.element.value,
        valid: true
      };
      var keyInfo = {
        keyName: e.key.toLowerCase(),
        modifierKey: this.isModifierKeyPressed(e)
      };

      var actionType = this.getActionType(keyInfo);
      var handler = this.getHandlerForAction(actionType);
      var newState = handler(currentState, keyInfo, this.options, this._history);

      if (!newState.valid) {
        this.options.invalidKeyCallback(e);
        e.preventDefault();
        return;
      }

      var shouldHandleValue = actionType !== _constants.ACTION_TYPES.UNKNOWN;
      if (!shouldHandleValue) {
        return;
      }

      e.preventDefault();

      var valueWithThousandsDelimiter = _helpers2.default.partialFormat(newState.value, this.options);
      var valueWithoutThousandsDelimiter = newState.value;

      this.element.value = valueWithThousandsDelimiter;
      this.element.rawValue = this.getRawValue(this.element.value);

      var offset = _helpers2.default.calculateOffset(valueWithoutThousandsDelimiter, valueWithThousandsDelimiter, newState.caretStart, this.options);
      var newCaretPos = newState.caretStart + offset;
      this.element.setSelectionRange(newCaretPos, newCaretPos);

      var shouldRecord = actionType !== _constants.ACTION_TYPES.UNDO && actionType !== _constants.ACTION_TYPES.REDO;
      if (shouldRecord) {
        this._history.addValue(valueWithThousandsDelimiter);
      }
    }
    /**
     * Backup event if input changes for any other reason, just format value
     * @param {e} Event
     */

  }, {
    key: 'onInput',
    value: function onInput(e) {
      this.setValue(this.element.value);
    }
    /**
     * Removes all listeners from the input
     */

  }, {
    key: 'removeListeners',
    value: function removeListeners() {
      for (var e in this._listeners) {
        this._listeners[e].element.removeEventListener(e, this._listeners[e].handler);
      }
    }
  }, {
    key: 'element',
    get: function get() {
      return this._element;
    }
  }, {
    key: 'options',
    get: function get() {
      return _extends({}, this._options);
    },
    set: function set(options) {
      this._options = _extends({}, this._options, options);
    }
  }]);

  return Finput;
}();

// Factory function


;
module.exports = exports['default'];